On Thu, May 25, 2023 at 01:15:07PM -0600, Khalid Aziz wrote:
diff --git a/mm/compaction.c b/mm/compaction.c
index 5a9501e0ae01..b548e05f0349 100644
--- a/mm/compaction.c
+++ b/mm/compaction.c
@@ -764,6 +764,42 @@ static bool too_many_isolated(pg_data_t *pgdat)
return too_many;
}
+/*
+ * Check if this base page should be skipped from isolation because
+ * it has extra refcounts that will prevent it from being migrated.
+ * This code is inspired by similar code in migrate_vma_check_page(),
+ * can_split_folio() and folio_migrate_mapping()
+ */
+static inline bool page_has_extra_refs(struct page *page,
+ struct address_space *mapping)
+{
+ unsigned long extra_refs;
+ struct folio *folio;
+
+ /*
+ * Skip this check for pages in ZONE_MOVABLE or MIGRATE_CMA
+ * pages that can not be long term pinned
+ */
+ if (is_zone_movable_page(page) || is_migrate_cma_page(page))
+ return false;
+
+ folio = page_folio(page);
+
+ /*
+ * caller holds a ref already from get_page_unless_zero()
+ * which is accounted for in folio_expected_refs()
+ */
+ extra_refs = folio_expected_refs(mapping, folio);
+
+ /*
+ * This is an admittedly racy check but good enough to determine
+ * if a page is pinned and can not be migrated
+ */
+ if ((folio_ref_count(folio) - extra_refs) > folio_mapcount(folio))
+ return true;
+ return false;
+}
+
/**
* isolate_migratepages_block() - isolate all migrate-able pages within
* a single pageblock
@@ -992,12 +1028,12 @@ isolate_migratepages_block(struct compact_control *cc, unsigned long low_pfn,
goto isolate_fail;
Just out of shot, we have ...
if (unlikely(!get_page_unless_zero(page)))
This is the perfect opportunity to use folio_get_nontail_page() instead.
You get back the folio without having to cast the pointer yourself
or call page_folio(). Now you can use a folio throughout your new
function, saving a call to compound_head().
For a followup patch, everything in this loop below this point can use
the folio ... that's quite a lot of change.
/*
* Migration will fail if an anonymous page is pinned in memory,
* so avoid taking lru_lock and isolating it unnecessarily in an
* admittedly racy check.
*/
Isn't "long term pinning" the wrong description of the problem? Long term
pins suggest to me FOLL_LONGTERM. I think this is simple short term
@@ -992,12 +1028,12 @@ isolate_migratepages_block(struct compact_control *cc, unsigned long low_pfn,
the folio ... that's quite a lot of change.
/*
- * Migration will fail if an anonymous page is pinned in memory,
- * so avoid taking lru_lock and isolating it unnecessarily in an
- * admittedly racy check.
+ * Migration will fail if a page has extra refcounts
+ * from long term pinning preventing it from migrating,
+ * so avoid taking lru_lock and isolating it unnecessarily.
*/
Isn't "long term pinning" the wrong description of the problem? Long term
pins that we care about here.